/* https://github.com/vuejs/vitepress/blob/main/src/client/theme-default/styles/vars.css */
:root {
    --vp-home-hero-name-color: #2C68C0;
    --vp-home-hero-name-color: transparent;
    --vp-home-hero-name-background: -webkit-linear-gradient(15deg, #2C68C0, #5DAC41);

    --vp-c-brand-1: #2C68C0;
    --vp-c-brand-2: #5DAC41;
}

.home-wrapper {
    margin: 0 auto;
    position: relative;
    width: 100%;
    padding: 32px 48px 96px;
    max-width: 1152px;
}
